package net.psammead.mwapi.ui.action;

import net.psammead.mwapi.MediaWiki;
import net.psammead.mwapi.NameSpace;
import net.psammead.mwapi.connection.Connection;

public abstract class UiActionBase extends ActionBase {
	protected UiActionBase(MediaWiki mediaWiki, Connection connection) {
		super(mediaWiki, connection);
	}
	
	/** returns whether a message exists in the site configuration */
	protected boolean messageAvailable(String messageName) {
		return site.message(messageName) != null;
	}
	
	/** returns a regexp for a named message, or null when not available */
	protected String messageRegexp(String messageName) {
		 String  raw = site.message(messageName);
		 if (raw == null)    return null;
		 return messageToRegexp(raw);
	}
	
    /** compile a mediawiki message from Allmessages.php into a regexp */
    private String messageToRegexp(String message) {
        return  // intro
                "(?s).*?" + 
                // remove spaces from both ends
                message.trim()
                // ULGY: remove everything after first LF -- this could solve the problem
                // that confirmrecreate looped endlessly when matching for a session loss
                .replaceAll("\n.*", "")
                // convert HTML tags, $1-like, {{}}-like, [[]] like, meta characters, CR, LF and TAB to .*?
                .replaceAll("(<.*?>|\\$\\d+|\\[\\[.*?\\]\\]|\\{\\{.*?\\}\\}|\\r|\\n|\\t|\\.|\\*|\\?|\\^|\\$|\\(|\\)|\\[|\\]|\\\\)+", ".*?")
                // convert multiple .?* to a single one
                .replaceAll("\\.\\*\\?(\\s*\\.\\*\\?)+", ".*?")
                // remove .*? from start and end
                .replaceAll("^\\.\\*\\?", "").replaceAll("\\.\\*\\?$", "") +
                // outro
                ".*";
                /*
                // replace $0-$n with .*? groups
                .replaceAll("\\$\\d+", "(.*?)");
                */
    }
	
    /** returns the localized title of a SpecialPage */
	protected String specialPage(String specialPage) {
		String	localized	= site.specialPageNames.get(specialPage);
		// TODO: use a better Exception
		if (localized == null)	throw new RuntimeException("specialPage not localized: " + specialPage + " in site: " + site);	
		return site.nameSpace(NameSpace.SPECIAL).addTo(localized);
	}
}
