package net.psammead.mwapi.ui.action.response;

import java.util.*;

import net.psammead.mwapi.*;

/** contains a chain of ResponseHandlers which are tried one after another until one returns true */
public class ResponseSelect implements ResponseHandler {
	private List<ResponseHandler>	handlers;

	public ResponseSelect() {
		handlers = new ArrayList<ResponseHandler>();
	}
	
	public void register(int responseCode, ResponseHandler handler) {
		register(new ResponsePattern(responseCode), handler);
	}

	public void register(int responseCode, String messageRegexp, ResponseHandler handler) {
		register(new ResponsePattern(responseCode, messageRegexp), handler);
	}
	
	public void register(ResponsePattern pattern, ResponseHandler handler) {
		register(new ResponseOption(pattern, handler));
	}
	
	public void register(ResponseHandler option) {
		handlers.add(option);
	}

	/** ResponseHandler implementation matches previously added handlers and executes the first matching */
	public boolean handle(ResponseData data) throws MediaWikiException {
		for (Iterator<ResponseHandler> it = handlers.iterator(); it.hasNext();) {
			ResponseHandler handler = it.next();
			if (handler.handle(data)) {
				return true;
			}
		}
		return false;
	}
}